@@ -1,3 +1,5 @@
+#include "cli.h"
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
@@ -15,7 +17,6 @@
 #include "log.h"
 #include "log_funcs.h"
 #include "call.h"
-#include "cli.h"
 #include "socket.h"
 #include "redis.h"
 #include "control_ng.h"
@@ -31,9 +32,6 @@
 #include "ssrc.h"
 #include "codec.h"
 
-#include "rtpengine_config.h"
-
-
 typedef void (*cli_handler_func)(str *, struct cli_writer *);
 typedef struct {
 	const char *cmd;
@@ -197,7 +195,7 @@ static const cli_handler_t cli_params_handlers[] = {
 };
 
 
-static void cli_list_call_info(struct cli_writer *cw, struct call *c);
+static void cli_list_call_info(struct cli_writer *cw, call_t *c);
 static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml);
 
 
@@ -226,8 +224,7 @@ static void cli_handler_do(const cli_handler_t *handlers, str *instr,
 
 static void destroy_own_foreign_calls(bool foreign_call, unsigned int uint_keyspace_db) {
 	struct call_monologue *ml = NULL;
-	GQueue call_list = G_QUEUE_INIT;
-	GList *i;
+	call_q calls = TYPED_GQUEUE_INIT;
 
 	ITERATE_CALL_LIST_START(CALL_ITERATOR_MAIN, c);
 		// match foreign_call flag
@@ -244,15 +241,15 @@ static void destroy_own_foreign_calls(bool foreign_call, unsigned int uint_keysp
 		obj_get(c);
 
 		// save call reference
-		g_queue_push_tail(&call_list, c);
+		t_queue_push_tail(&calls, c);
 next:;
 	ITERATE_CALL_LIST_NEXT_END(c);
 
 	// destroy calls
-	struct call *c = NULL;
-	while ((c = g_queue_pop_head(&call_list))) {
+	call_t *c = NULL;
+	while ((c = t_queue_pop_head(&calls))) {
 		if (!c->ml_deleted) {
-			for (i = c->monologues.head; i; i = i->next) {
+			for (__auto_type i = c->monologues.head; i; i = i->next) {
 				ml = i->data;
 				gettimeofday(&(ml->terminated), NULL);
 				ml->term_reason = FORCED;
@@ -286,7 +283,6 @@ static void cli_endpoints_print(struct cli_writer *cw, const GQueue *q, const ch
 
 static void cli_incoming_params_start(str *instr, struct cli_writer *cw) {
 	int count = 0;
-	GList *s;
 	struct intf_config *ifa;
 
 	for (unsigned int i = 0; i < num_log_levels; i++)
@@ -302,7 +298,8 @@ static void cli_incoming_params_start(str *instr, struct cli_writer *cw) {
 			"redis_allowed_errors = %d\nredis_disable_time = %d\nredis_cmd_timeout = %d\nredis_connect_timeout = %d\n"
 			"max-cpu = %.1f\n"
 			"max-load = %.2f\n"
-			"max-bandwidth = %" PRIu64 "\n",
+			"max-bandwidth = %" PRIu64 "\n"
+			"max-recv-iters = %d\n",
 			initial_rtpe_config.kernel_table, initial_rtpe_config.max_sessions,
 			initial_rtpe_config.timeout, initial_rtpe_config.silent_timeout, initial_rtpe_config.final_timeout,
 			initial_rtpe_config.offer_timeout, initial_rtpe_config.delete_delay,
@@ -316,15 +313,16 @@ static void cli_incoming_params_start(str *instr, struct cli_writer *cw) {
 			initial_rtpe_config.redis_disable_time, initial_rtpe_config.redis_cmd_timeout, initial_rtpe_config.redis_connect_timeout,
 			(double) initial_rtpe_config.cpu_limit / 100,
 			(double) initial_rtpe_config.load_limit / 100,
-			initial_rtpe_config.bw_limit);
+			initial_rtpe_config.bw_limit,
+			initial_rtpe_config.max_recv_iters);
 
-	for(s = initial_rtpe_config.interfaces.head; s ; s = s->next) {
+	for (__auto_type s = initial_rtpe_config.interfaces.head; s ; s = s->next) {
 		ifa = s->data;
 		cw->cw_printf(cw,"interface[%d] = %s\\%s \n", count, ifa->name.s, sockaddr_print_buf(&(ifa->local_address.addr)));
 		++count;
 	}
 	count=0;
-	for (s = initial_rtpe_config.redis_subscribed_keyspaces.head; s ; s = s->next) {
+	for (__auto_type s = initial_rtpe_config.redis_subscribed_keyspaces.head; s ; s = s->next) {
 		cw->cw_printf(cw,"keyspace[%d] = %d \n", count, GPOINTER_TO_UINT(s->data));
 		++count;
 	}
@@ -341,7 +339,6 @@ static void cli_incoming_params_start(str *instr, struct cli_writer *cw) {
 
 static void cli_incoming_params_current(str *instr, struct cli_writer *cw) {
 	int count = 0;
-	GList *c;
 	struct intf_config *ifa;
 
 	for (unsigned int i = 0; i < num_log_levels; i++)
@@ -357,7 +354,8 @@ static void cli_incoming_params_current(str *instr, struct cli_writer *cw) {
 			"redis_allowed_errors = %d\nredis_disable_time = %d\nredis_cmd_timeout = %d\nredis_connect_timeout = %d\n"
 			"max-cpu = %.1f\n"
 			"max-load = %.2f\n"
-			"max-bw = %" PRIu64 "\n",
+			"max-bw = %" PRIu64 "\n"
+			"max-recv-iters = %d\n",
 			rtpe_config.kernel_table, rtpe_config.max_sessions, rtpe_config.timeout,
 			rtpe_config.silent_timeout, rtpe_config.final_timeout, rtpe_config.offer_timeout,
 			rtpe_config.delete_delay, rtpe_config.redis_expires_secs, rtpe_config.default_tos,
@@ -368,15 +366,16 @@ static void cli_incoming_params_current(str *instr, struct cli_writer *cw) {
 			rtpe_config.redis_disable_time, rtpe_config.redis_cmd_timeout, rtpe_config.redis_connect_timeout,
 			(double) rtpe_config.cpu_limit / 100,
 			(double) rtpe_config.load_limit / 100,
-			rtpe_config.bw_limit);
+			rtpe_config.bw_limit,
+			rtpe_config.max_recv_iters);
 
-	for(c = rtpe_config.interfaces.head; c ; c = c->next) {
+	for (__auto_type c = rtpe_config.interfaces.head; c ; c = c->next) {
 		ifa = c->data;
 		cw->cw_printf(cw,"interface[%d] = %s\\%s \n", count, ifa->name.s, sockaddr_print_buf(&(ifa->local_address.addr)));
 		++count;
 	}
 	count=0;
-	for (c = rtpe_config.redis_subscribed_keyspaces.head; c ; c = c->next) {
+	for (__auto_type c = rtpe_config.redis_subscribed_keyspaces.head; c ; c = c->next) {
 		cw->cw_printf(cw,"keyspace[%d] = %d \n", count, GPOINTER_TO_UINT(c->data));
 		++count;
 	}
@@ -392,11 +391,11 @@ static void cli_incoming_params_current(str *instr, struct cli_writer *cw) {
 }
 
 #define int_diff_print(struct_member, option_string) \
-	if (initial_rtpe_config.struct_member != rtpe_config.struct_member) { \
+	if (initial_rtpe_config.struct_member != atomic_get_na(&rtpe_config.struct_member)) { \
 		if (strcmp(option, "diff") == 0) \
 			cw->cw_printf(cw, "%s: %lld => %lld\n", option_string, \
 			              (long long)initial_rtpe_config.struct_member, \
-			              (long long)rtpe_config.struct_member); \
+			              (long long)atomic_get_na(&rtpe_config.struct_member)); \
 		else if (strcmp(option, "revert") == 0) \
 			rtpe_config.struct_member = initial_rtpe_config.struct_member; \
 	}
@@ -407,7 +406,6 @@ static void cli_incoming_diff_or_revert(struct cli_writer *cw, char* option) {
 #include "loglevels.h"
 #undef ll
 
-	rwlock_lock_w(&rtpe_config.config_lock);
 	int_diff_print(max_sessions, "max-sessions");
 	int_diff_print(cpu_limit, "max-cpu");
 	int_diff_print(load_limit, "max-load");
@@ -420,7 +418,6 @@ static void cli_incoming_diff_or_revert(struct cli_writer *cw, char* option) {
 	int_diff_print(redis_disable_time, "redis_disable_time");
 	int_diff_print(redis_cmd_timeout, "redis_cmd_timeout");
 	int_diff_print(redis_connect_timeout, "redis_connect_timeout-db");
-	rwlock_unlock_w(&rtpe_config.config_lock);
 }
 
 static void cli_incoming_params_diff(str *instr, struct cli_writer *cw) {
@@ -437,33 +434,33 @@ static void cli_incoming_params_revert(str *instr, struct cli_writer *cw) {
 static void cli_incoming_list_counters(str *instr, struct cli_writer *cw) {
 	cw->cw_printf(cw, "\nCurrent per-second counters:\n\n");
 	cw->cw_printf(cw, " Packets per second (userspace)                  :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.packets_user));
+			atomic64_get_na(&rtpe_stats_rate.packets_user));
 	cw->cw_printf(cw, " Bytes per second (userspace)                    :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.bytes_user));
+			atomic64_get_na(&rtpe_stats_rate.bytes_user));
 	cw->cw_printf(cw, " Errors per second (userspace)                   :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.errors_user));
+			atomic64_get_na(&rtpe_stats_rate.errors_user));
 	cw->cw_printf(cw, " Packets per second (kernel)                     :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.packets_kernel));
+			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
 	cw->cw_printf(cw, " Bytes per second (kernel)                       :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.bytes_kernel));
+			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
 	cw->cw_printf(cw, " Errors per second (kernel)                      :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.errors_kernel));
+			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
 	cw->cw_printf(cw, " Packets per second (total)                      :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.packets_user) +
-			atomic64_get(&rtpe_stats_rate.packets_kernel));
+			atomic64_get_na(&rtpe_stats_rate.packets_user) +
+			atomic64_get_na(&rtpe_stats_rate.packets_kernel));
 	cw->cw_printf(cw, " Bytes per second (total)                        :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.bytes_user) +
-			atomic64_get(&rtpe_stats_rate.bytes_kernel));
+			atomic64_get_na(&rtpe_stats_rate.bytes_user) +
+			atomic64_get_na(&rtpe_stats_rate.bytes_kernel));
 	cw->cw_printf(cw, " Errors per second (total)                       :%" PRIu64 "\n",
-			atomic64_get(&rtpe_stats_rate.errors_user) +
-			atomic64_get(&rtpe_stats_rate.errors_kernel));
+			atomic64_get_na(&rtpe_stats_rate.errors_user) +
+			atomic64_get_na(&rtpe_stats_rate.errors_kernel));
 }
 
 static void cli_incoming_list_totals(str *instr, struct cli_writer *cw) {
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics(NULL));
+	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
 
-	for (GList *l = metrics->head; l; l = l->next) {
-		struct stats_metric *m = l->data;
+	for (__auto_type l = metrics->head; l; l = l->next) {
+		stats_metric *m = l->data;
 		if (!m->descr)
 			continue;
 		if (m->value_long) {
@@ -479,17 +476,17 @@ static void cli_incoming_list_totals(str *instr, struct cli_writer *cw) {
 
 static void cli_incoming_list_numsessions(str *instr, struct cli_writer *cw) {
        rwlock_lock_r(&rtpe_callhash_lock);
-       cw->cw_printf(cw, "Current sessions own: "UINT64F"\n", g_hash_table_size(rtpe_callhash) - atomic64_get(&rtpe_stats_gauge.foreign_sessions));
-       cw->cw_printf(cw, "Current sessions foreign: "UINT64F"\n", atomic64_get(&rtpe_stats_gauge.foreign_sessions));
-       cw->cw_printf(cw, "Current sessions total: %i\n", g_hash_table_size(rtpe_callhash));
+       cw->cw_printf(cw, "Current sessions own: "UINT64F"\n", t_hash_table_size(rtpe_callhash) - atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+       cw->cw_printf(cw, "Current sessions foreign: "UINT64F"\n", atomic64_get_na(&rtpe_stats_gauge.foreign_sessions));
+       cw->cw_printf(cw, "Current sessions total: %i\n", t_hash_table_size(rtpe_callhash));
        rwlock_unlock_r(&rtpe_callhash_lock);
-       cw->cw_printf(cw, "Current transcoded media: "UINT64F"\n", atomic64_get(&rtpe_stats_gauge.transcoded_media));
+       cw->cw_printf(cw, "Current transcoded media: "UINT64F"\n", atomic64_get_na(&rtpe_stats_gauge.transcoded_media));
        cw->cw_printf(cw, "Current sessions ipv4 only media: " UINT64F "\n",
-		       atomic64_get(&rtpe_stats_gauge.ipv4_sessions));
+		       atomic64_get_na(&rtpe_stats_gauge.ipv4_sessions));
        cw->cw_printf(cw, "Current sessions ipv6 only media: " UINT64F "\n",
-		       atomic64_get(&rtpe_stats_gauge.ipv6_sessions));
+		       atomic64_get_na(&rtpe_stats_gauge.ipv6_sessions));
        cw->cw_printf(cw, "Current sessions ip mixed  media: " UINT64F "\n",
-		       atomic64_get(&rtpe_stats_gauge.mixed_sessions));
+		       atomic64_get_na(&rtpe_stats_gauge.mixed_sessions));
 }
 
 static void cli_incoming_list_maxsessions(str *instr, struct cli_writer *cw) {
@@ -538,48 +535,20 @@ static void cli_incoming_list_maxopenfiles(str *instr, struct cli_writer *cw) {
 }
 
 static void cli_incoming_list_timeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-
-	/* don't lock anything while reading the value */
 	cw->cw_printf(cw, "TIMEOUT=%u\n", rtpe_config.timeout);
-
-	rwlock_unlock_r(&rtpe_config.config_lock);
-
-	return ;
 }
 static void cli_incoming_list_silenttimeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-
-	/* don't lock anything while reading the value */
 	cw->cw_printf(cw, "SILENT_TIMEOUT=%u\n", rtpe_config.silent_timeout);
-
-	rwlock_unlock_r(&rtpe_config.config_lock);
-
-	return ;
 }
 static void cli_incoming_list_finaltimeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-
-	/* don't lock anything while reading the value */
 	cw->cw_printf(cw, "FINAL_TIMEOUT=%u\n", rtpe_config.final_timeout);
-
-	rwlock_unlock_r(&rtpe_config.config_lock);
-
-	return ;
 }
 static void cli_incoming_list_offertimeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-
-	/* don't lock anything while reading the value */
 	cw->cw_printf(cw, "OFFER_TIMEOUT=%u\n", rtpe_config.offer_timeout);
-
-	rwlock_unlock_r(&rtpe_config.config_lock);
-
-	return ;
 }
 
 static void cli_incoming_list_callid(str *instr, struct cli_writer *cw) {
-	struct call *c = 0;
+	call_t *c = 0;
 
 	if (instr->len == 0) {
 		cw->cw_printf(cw, "%s\n", "More parameters required.");
@@ -600,18 +569,28 @@ static void cli_incoming_list_callid(str *instr, struct cli_writer *cw) {
 }
 
 
-static void cli_list_call_info(struct cli_writer *cw, struct call *c) {
+static void cli_list_call_info(struct cli_writer *cw, call_t *c) {
 	struct call_monologue *ml;
-	GList *l;
 
 	cw->cw_printf(cw,
-			 "\ncallid: %s\ndeletionmark: %s\ncreated: %i\nproxy: %s\ntos: %u\nlast_signal: %llu\n"
-			 "redis_keyspace: %i\nforeign: %s\n\n",
+			 "\n"
+			 "callid: %s\n"
+			 "deletionmark: %s\n"
+			 "created: %i\n"
+			 "proxy: %s\n"
+			 "tos: %u\n"
+			 "last_signal: %llu\n"
+			 "redis_keyspace: %i\n"
+			 "last redis update: %llu\n"
+			 "foreign: %s\n"
+			 "recording: %s\n"
+			 "\n",
 			 c->callid.s, c->ml_deleted ? "yes" : "no", (int) c->created.tv_sec, c->created_from,
 			 (unsigned int) c->tos, (unsigned long long) c->last_signal, c->redis_hosted_db,
-			 IS_FOREIGN_CALL(c) ? "yes" : "no");
+			 (unsigned long long) atomic64_get_na(&c->last_redis_update),
+			 IS_FOREIGN_CALL(c) ? "yes" : "no", c->recording ? "yes" : "no");
 
-	for (l = c->monologues.head; l; l = l->next) {
+	for (__auto_type l = c->monologues.head; l; l = l->next) {
 		ml = l->data;
 		cli_list_tag_info(cw, ml);
 	}
@@ -643,17 +622,33 @@ static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml)
 		tim_result_duration.tv_sec,
 		tim_result_duration.tv_usec);
 
-	for (GList *sub = ml->subscriptions.head; sub; sub = sub->next) {
-		struct call_subscription *cs = sub->data;
-		struct call_monologue *csm = cs->monologue;
-		cw->cw_printf(cw, "---     subscribed to '" STR_FORMAT_M "'\n",
-				STR_FMT_M(&csm->tag));
-	}
-	for (GList *sub = ml->subscribers.head; sub; sub = sub->next) {
-		struct call_subscription *cs = sub->data;
-		struct call_monologue *csm = cs->monologue;
-		cw->cw_printf(cw, "---     subscription of '" STR_FORMAT_M "'\n",
-				STR_FMT_M(&csm->tag));
+	for (int i = 0; i < ml->medias->len; i++)
+	{
+		struct call_media * media = ml->medias->pdata[i];
+		if (!media)
+			continue;
+
+		for (__auto_type sub = media->media_subscriptions.head; sub; sub = sub->next)
+		{
+			struct media_subscription * ms = sub->data;
+			struct call_media * sub_media = ms->media;
+			if (!sub_media)
+				continue;
+
+			cw->cw_printf(cw, "---     subscribed to media with monologue tag '" STR_FORMAT_M "' (index: %d)\n",
+					STR_FMT_M(&ms->monologue->tag), sub_media->index);
+		}
+
+		for (__auto_type sub = media->media_subscribers.head; sub; sub = sub->next)
+		{
+			struct media_subscription * ms = sub->data;
+			struct call_media * sub_media = ms->media;
+			if (!sub_media)
+				continue;
+
+			cw->cw_printf(cw, "---     subscription of media with monologue tag '" STR_FORMAT_M "' (index: %d)\n",
+					STR_FMT_M(&ms->monologue->tag), sub_media->index);
+		}
 	}
 
 	for (unsigned int k = 0; k < ml->medias->len; k++) {
@@ -661,7 +656,7 @@ static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml)
 		if (!md)
 			continue;
 
-		const struct rtp_payload_type *rtp_pt = __rtp_stats_codec(md);
+		const rtp_payload_type *rtp_pt = __rtp_stats_codec(md);
 
 		cw->cw_printf(cw, "------ Media #%u (" STR_FORMAT " over %s) using ",
 				md->index,
@@ -672,7 +667,7 @@ static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml)
 		else
 			cw->cw_printf(cw, STR_FORMAT "\n", STR_FMT(&rtp_pt->encoding_with_params));
 
-		for (GList *o = md->streams.head; o; o = o->next) {
+		for (__auto_type o = md->streams.head; o; o = o->next) {
 			ps = o->data;
 
 			if (PS_ISSET(ps, FALLBACK_RTCP))
@@ -682,23 +677,19 @@ static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml)
 			local_addr = sockaddr_print_buf(&local_endpoint->address);
 
 			cw->cw_printf(cw, "-------- Port %15s:%-5u <> %15s:%-5u%s, SSRC %" PRIx32 ", "
-					 "" UINT64F " p, " UINT64F " b, " UINT64F " e, " UINT64F " ts",
+					 "" UINT64F " p, " UINT64F " b, " UINT64F " e, " UINT64F " uts "
+					 UINT64F " kts",
 					 local_addr,
 					 (unsigned int) local_endpoint->port,
 					 sockaddr_print_buf(&ps->endpoint.address),
 					 ps->endpoint.port,
 					 (!PS_ISSET(ps, RTP) && PS_ISSET(ps, RTCP)) ? " (RTCP)" : "",
 					 ps->ssrc_in[0] ? ps->ssrc_in[0]->parent->h.ssrc : 0,
-					 atomic64_get(&ps->stats_in.packets),
-					 atomic64_get(&ps->stats_in.bytes), atomic64_get(&ps->stats_in.errors),
-					 atomic64_get(&ps->last_packet));
-#if RE_HAS_MEASUREDELAY
-			if (PS_ISSET(ps, RTP) || !PS_ISSET(ps, RTCP))
-				cw->cw_printf(cw, ", %.9f delay_min, %.9f delay_avg, %.9f delay_max",
-						 (double) ps->stats.delay_min / 1000000,
-						 (double) ps->stats.delay_avg / 1000000,
-						 (double) ps->stats.delay_max / 1000000);
-#endif
+					 atomic64_get_na(&ps->stats_in->packets),
+					 atomic64_get_na(&ps->stats_in->bytes),
+					 atomic64_get_na(&ps->stats_in->errors),
+					 atomic64_get_na(&ps->last_packet),
+					 atomic64_get_na(&ps->stats_in->last_packet));
 			cw->cw_printf(cw, "\n");
 		}
 	}
@@ -706,51 +697,67 @@ static void cli_list_tag_info(struct cli_writer *cw, struct call_monologue *ml)
 
 
 static void cli_incoming_list_sessions(str *instr, struct cli_writer *cw) {
-	size_t found = 0;
-	bool all = false, own = false;
+	bool found = false;
+	enum { all, own, foreign, recording } which = -1;
 
-	static const char* LIST_ALL = "all";
-	static const char* LIST_OWN = "own";
-	static const char* LIST_FOREIGN = "foreign";
+	static const char *keywords[] = {
+		[all] = "all",
+		[own] = "own",
+		[foreign] = "foreign",
+		[recording] = "recording",
+	};
 
 	if (str_shift(instr, 1)) {
 		cw->cw_printf(cw, "%s\n", "More parameters required.");
 		return;
 	}
 
-	if (str_cmp(instr, LIST_ALL) == 0)
-		all = true;
-	else if (str_cmp(instr, LIST_OWN) == 0)
-		own = true;
-	else if (str_cmp(instr, LIST_FOREIGN) == 0)
-	{ } // default
-	else {
+	for (unsigned int i = 0; i < G_N_ELEMENTS(keywords); i++) {
+		if (str_cmp(instr, keywords[i]) == 0) {
+			which = i;
+			break;
+		}
+	}
+	if (which == -1) {
 		// list session for callid
 		cli_incoming_list_callid(instr, cw);
 		return;
 	}
 
 	ITERATE_CALL_LIST_START(CALL_ITERATOR_MAIN, call);
-		if (!all) {
-			if (IS_FOREIGN_CALL(call) && own)
-				goto next;
-			if (!IS_FOREIGN_CALL(call) && !own)
-				goto next;
+		switch (which) {
+			case all:
+				break;
+			case foreign:
+				if (!IS_FOREIGN_CALL(call))
+					goto next;
+				break;
+			case own:
+				if (IS_FOREIGN_CALL(call))
+					goto next;
+				break;
+			case recording:
+				if (!call->recording)
+					goto next;
+				break;
 		}
-		found++;
+		found = true;
 
-		cw->cw_printf(cw, "callid: %60s | deletionmark:%4s | created:%12i | proxy:%s | redis_keyspace:%i | foreign:%s\n", call->callid.s, call->ml_deleted?"yes":"no", (int)call->created.tv_sec, call->created_from, call->redis_hosted_db, IS_FOREIGN_CALL(call)?"yes":"no");
+		cw->cw_printf(cw, "ID: %60s | del:%s | creat:%12li | prx:%s | redis:%2i | frgn:%s | rec:%s\n",
+				call->callid.s, call->ml_deleted ? "y" : "n",
+				(long) call->created.tv_sec,
+				call->created_from, call->redis_hosted_db,
+				IS_FOREIGN_CALL(call) ? "y" : "n",
+				call->recording ? "y" : "n");
 
 next:;
 	ITERATE_CALL_LIST_NEXT_END(call);
 
 	if (!found) {
-		if (all)
+		if (which == all)
 			cw->cw_printf(cw, "No sessions on this media relay.\n");
-		else if (own)
-			cw->cw_printf(cw, "No own sessions on this media relay.\n");
 		else
-			cw->cw_printf(cw, "No foreign sessions on this media relay.\n");
+			cw->cw_printf(cw, "No %s sessions on this media relay.\n", keywords[which]);
 	}
 
 	return;
@@ -812,14 +819,10 @@ static void cli_incoming_set_maxsessions(str *instr, struct cli_writer *cw) {
 	} else if (maxsessions_num < disabled) {
 		cw->cw_printf(cw,  "Fail setting maxsessions to %ld; either positive or -1 values allowed\n", maxsessions_num);
 	} else if (maxsessions_num == disabled) {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		rtpe_config.max_sessions = maxsessions_num;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(&rtpe_config.max_sessions, maxsessions_num);
 		cw->cw_printf(cw,  "Success setting maxsessions to %ld; disable feature\n", maxsessions_num);
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		rtpe_config.max_sessions = maxsessions_num;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(&rtpe_config.max_sessions, maxsessions_num);
 		cw->cw_printf(cw,  "Success setting maxsessions to %ld\n", maxsessions_num);
 	}
 
@@ -845,9 +848,7 @@ static void cli_incoming_set_maxcpu(str *instr, struct cli_writer *cw) {
 		cw->cw_printf(cw,  "Fail setting maxcpu to %s; no digists found\n", instr->s);
 		return;
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		rtpe_config.cpu_limit = num * 100;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(&rtpe_config.cpu_limit, (int) (num * 100));
 		cw->cw_printf(cw,  "Success setting maxcpu to %.1f\n", num);
 	}
 
@@ -872,9 +873,7 @@ static void cli_incoming_set_maxload(str *instr, struct cli_writer *cw) {
 		cw->cw_printf(cw,  "Fail setting maxload to %s; no digists found\n", instr->s);
 		return;
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		rtpe_config.load_limit = num * 100;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(&rtpe_config.load_limit, (int) (num * 100));
 		cw->cw_printf(cw,  "Success setting maxload to %.2f\n", num);
 	}
 
@@ -899,9 +898,7 @@ static void cli_incoming_set_maxbw(str *instr, struct cli_writer *cw) {
 		cw->cw_printf(cw,  "Fail setting maxbw to %s; no digists found\n", instr->s);
 		return;
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		rtpe_config.bw_limit = num * 100;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(&rtpe_config.bw_limit, num);
 		cw->cw_printf(cw,  "Success setting maxbw to %" PRIu64 "\n", num);
 	}
 
@@ -927,9 +924,7 @@ static void cli_incoming_set_gentimeout(str *instr, struct cli_writer *cw, int *
 		cw->cw_printf(cw,  "Fail setting timeout to %s; no digists found\n", instr->s);
 		return;
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
-		*conf_timeout = (int) timeout_num;
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		atomic_set_na(conf_timeout, timeout_num);
 		cw->cw_printf(cw,  "Success setting timeout to %lu\n", timeout_num);
 	}
 }
@@ -974,9 +969,8 @@ static void cli_incoming_params(str *instr, struct cli_writer *cw) {
 }
 
 static void cli_incoming_terminate(str *instr, struct cli_writer *cw) {
-   struct call* c=0;
+   call_t * c=0;
    struct call_monologue *ml;
-   GList *i;
 
    if (str_shift(instr, 1)) {
        cw->cw_printf(cw, "%s\n", "More parameters required.");
@@ -1029,7 +1023,7 @@ static void cli_incoming_terminate(str *instr, struct cli_writer *cw) {
    }
 
    if (!c->ml_deleted) {
-	   for (i = c->monologues.head; i; i = i->next) {
+	   for (__auto_type i = c->monologues.head; i; i = i->next) {
 		   ml = i->data;
 		   gettimeofday(&(ml->terminated), NULL);
 		   ml->term_reason = FORCED;
@@ -1062,7 +1056,7 @@ static void cli_incoming_ksadd(str *instr, struct cli_writer *cw) {
 	} else if (endptr == instr->s) {
 		cw->cw_printf(cw, "Fail adding keyspace %s to redis notifications; no digists found\n", instr->s);
 	} else {
-		rwlock_lock_w(&rtpe_config.config_lock);
+		rwlock_lock_w(&rtpe_config.keyspaces_lock);
 		if (!g_queue_find(&rtpe_config.redis_subscribed_keyspaces, GUINT_TO_POINTER(uint_keyspace_db))) {
 			g_queue_push_tail(&rtpe_config.redis_subscribed_keyspaces, GUINT_TO_POINTER(uint_keyspace_db));
 			redis_notify_subscribe_action(rtpe_redis_notify, SUBSCRIBE_KEYSPACE, uint_keyspace_db);
@@ -1070,7 +1064,7 @@ static void cli_incoming_ksadd(str *instr, struct cli_writer *cw) {
 		} else {
 			cw->cw_printf(cw, "Keyspace %lu is already among redis notifications.\n", uint_keyspace_db);
 		}
-		rwlock_unlock_w(&rtpe_config.config_lock);
+		rwlock_unlock_w(&rtpe_config.keyspaces_lock);
 	}
 }
 
@@ -1087,7 +1081,7 @@ static void cli_incoming_ksrm(str *instr, struct cli_writer *cw) {
 	errno = 0;
 	uint_keyspace_db = strtoul(instr->s, &endptr, 10);
 
-	rwlock_lock_w(&rtpe_config.config_lock);
+	rwlock_lock_w(&rtpe_config.keyspaces_lock);
 	if ((errno == ERANGE && (uint_keyspace_db == ULONG_MAX)) || (errno != 0 && uint_keyspace_db == 0)) {
 		cw->cw_printf(cw, "Fail removing keyspace %s to redis notifications; errono=%d\n", instr->s, errno);
         } else if (endptr == instr->s) {
@@ -1106,7 +1100,7 @@ static void cli_incoming_ksrm(str *instr, struct cli_writer *cw) {
 	} else {
 		cw->cw_printf(cw, "Keyspace %lu is not among redis notifications.\n", uint_keyspace_db);
 	}
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	rwlock_unlock_w(&rtpe_config.keyspaces_lock);
 
 }
 
@@ -1115,11 +1109,11 @@ static void cli_incoming_kslist(str *instr, struct cli_writer *cw) {
 
 	cw->cw_printf(cw,  "\nSubscribed-on keyspaces:\n");
     
-	rwlock_lock_r(&rtpe_config.config_lock);
+	rwlock_lock_r(&rtpe_config.keyspaces_lock);
 	for (l = rtpe_config.redis_subscribed_keyspaces.head; l; l = l->next) {
 		cw->cw_printf(cw,  "%u ", GPOINTER_TO_UINT(l->data));
 	}
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	rwlock_unlock_r(&rtpe_config.keyspaces_lock);
 
 	cw->cw_printf(cw, "\n");
 }
@@ -1154,7 +1148,7 @@ static void cli_incoming_debug(str *instr, struct cli_writer *cw) {
 	}
 
 	str callid;
-	if (str_token_sep(&callid, instr, ' '))
+	if (!str_token_sep(&callid, instr, ' '))
 		callid = STR_NULL;
 
 	if (!callid.len) {
@@ -1175,7 +1169,7 @@ static void cli_incoming_debug(str *instr, struct cli_writer *cw) {
 		}
 	}
 
-	struct call *c = call_get(&callid);
+	call_t *c = call_get(&callid);
 
 	if (!c) {
 		cw->cw_printf(cw, "Call ID '" STR_FORMAT "' not found\n", STR_FMT(&callid));
@@ -1254,7 +1248,6 @@ struct cli *cli_new(const endpoint_t *ep) {
       goto fail;
    }
 
-   obj_put(c);
    return c;
 
 fail:
@@ -1292,7 +1285,7 @@ static void cli_incoming_set_loglevel(str *instr, struct cli_writer *cw) {
 
 	str subsys = STR_NULL;
 	if (instr->len && (instr->s[0] < '0' || instr->s[0] > '9'))
-		if (str_token_sep(&subsys, instr, ' '))
+		if (!str_token_sep(&subsys, instr, ' '))
 			subsys = STR_NULL;
 
 	if (!instr->len) {
@@ -1313,9 +1306,7 @@ static void cli_incoming_set_loglevel(str *instr, struct cli_writer *cw) {
 }
 
 static void cli_incoming_list_redisallowederrors(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.redis_allowed_errors);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.redis_allowed_errors));
 }
 
 static void cli_incoming_set_redisallowederrors(str *instr, struct cli_writer *cw) {
@@ -1330,17 +1321,13 @@ static void cli_incoming_set_redisallowederrors(str *instr, struct cli_writer *c
 	errno = 0;
 	allowed_errors = strtol(instr->s, &endptr, 10);
 
-	rwlock_lock_w(&rtpe_config.config_lock);
-	rtpe_config.redis_allowed_errors = allowed_errors;
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	atomic_set_na(&rtpe_config.redis_allowed_errors, allowed_errors);
 
 	cw->cw_printf(cw,  "Success setting redis-allowed-errors to %ld\n", allowed_errors);
 }
 
 static void cli_incoming_list_redisdisabletime(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.redis_disable_time);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.redis_disable_time));
 }
 
 static void cli_incoming_set_redisdisable(str *instr, struct cli_writer *cw) {
@@ -1399,17 +1386,13 @@ static void cli_incoming_set_redisdisabletime(str *instr, struct cli_writer *cw)
 		return;
 	}
 
-	rwlock_lock_w(&rtpe_config.config_lock);
-	rtpe_config.redis_disable_time = seconds;
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	atomic_set_na(&rtpe_config.redis_disable_time, seconds);
 
 	cw->cw_printf(cw,  "Success setting redis-disable-time to %ld\n", seconds);
 }
 
 static void cli_incoming_list_redisconnecttimeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.redis_connect_timeout);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.redis_connect_timeout));
 }
 
 static void cli_incoming_set_redisconnecttimeout(str *instr, struct cli_writer *cw) {
@@ -1427,16 +1410,12 @@ static void cli_incoming_set_redisconnecttimeout(str *instr, struct cli_writer *
 		cw->cw_printf(cw,  "Invalid redis-connect-timeout value %ld, must be > 0\n", timeout);
 		return;
 	}
-	rwlock_lock_w(&rtpe_config.config_lock);
-	rtpe_config.redis_connect_timeout = timeout;
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	atomic_set_na(&rtpe_config.redis_connect_timeout, timeout);
 	cw->cw_printf(cw,  "Success setting redis-connect-timeout to %ld\n", timeout);
 }
 
 static void cli_incoming_list_deletedelay(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.delete_delay);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.delete_delay));
 }
 
 static void cli_incoming_set_deletedelay(str *instr, struct cli_writer *cw) {
@@ -1450,9 +1429,7 @@ static void cli_incoming_set_deletedelay(str *instr, struct cli_writer *cw) {
 		cw->cw_printf(cw, "Invalid delete-delay value\n");
 		return;
 	}
-	rwlock_lock_w(&rtpe_config.config_lock);
-	rtpe_config.delete_delay = seconds;
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	atomic_set_na(&rtpe_config.delete_delay, seconds);
 	cw->cw_printf(cw, "Success setting delete-delay to %d\n", seconds);
 }
 
@@ -1463,7 +1440,7 @@ static void cli_incoming_call(str *instr, struct cli_writer *cw) {
 	}
 
 	str callid;
-	if (str_token_sep(&callid, instr, ' '))
+	if (!str_token_sep(&callid, instr, ' '))
 		callid = STR_NULL;
 
 	if (!callid.len) {
@@ -1481,8 +1458,7 @@ static void cli_incoming_call(str *instr, struct cli_writer *cw) {
 
 	if (cw->call) {
 		rwlock_unlock_w(&cw->call->master_lock);
-		obj_put(cw->call);
-		cw->call = NULL;
+		obj_release(cw->call);
 	}
 }
 
@@ -1496,8 +1472,7 @@ static void cli_incoming_call_terminate(str *instr, struct cli_writer *cw) {
 	ilog(LOG_WARN, "Call " STR_FORMAT_M " terminated by operator", STR_FMT_M(&cw->call->callid));
 	rwlock_unlock_w(&cw->call->master_lock);
 	call_destroy(cw->call);
-	obj_put(cw->call);
-	cw->call = NULL;
+	obj_release(cw->call);
 }
 static void cli_incoming_call_debug(str *instr, struct cli_writer *cw) {
 	str_shift(instr, 1);
@@ -1527,7 +1502,7 @@ static void cli_incoming_call_tag(str *instr, struct cli_writer *cw) {
 	}
 
 	str tag;
-	if (str_token_sep(&tag, instr, ' '))
+	if (!str_token_sep(&tag, instr, ' '))
 		tag = STR_NULL;
 
 	if (!tag.len) {
@@ -1597,9 +1572,7 @@ static void cli_incoming_tag_detdtmf(str *instr, struct cli_writer *cw) {
 
 
 static void cli_incoming_list_rediscmdtimeout(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.redis_cmd_timeout);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.redis_cmd_timeout));
 }
 
 static void cli_incoming_set_rediscmdtimeout(str *instr, struct cli_writer *cw) {
@@ -1620,15 +1593,11 @@ static void cli_incoming_set_rediscmdtimeout(str *instr, struct cli_writer *cw)
 		return;
 	}
 
-	rwlock_lock_w(&rtpe_config.config_lock);
-	if (rtpe_config.redis_cmd_timeout == timeout) {
-		rwlock_unlock_w(&rtpe_config.config_lock);
+	if (atomic_get_na(&rtpe_config.redis_cmd_timeout) == timeout) {
 		cw->cw_printf(cw,  "Success setting redis-cmd-timeout to %ld\n", timeout);
 		return;
 	}
-	rtpe_config.redis_cmd_timeout = timeout;
-	rwlock_unlock_w(&rtpe_config.config_lock);
-
+	atomic_set_na(&rtpe_config.redis_cmd_timeout, timeout);
 
 	if (timeout == 0) {
 		cw->cw_printf(cw, "Warning: Setting redis-cmd-timeout to 0 (no timeout) will require a redis reconnect\n");
@@ -1679,14 +1648,41 @@ static void cli_incoming_list_interfaces(str *instr, struct cli_writer *cw) {
 		unsigned int r = lif->spec->port_pool.max - lif->spec->port_pool.min + 1;
 		cw->cw_printf(cw, " Ports used: %5u / %5u (%5.1f%%)\n",
 				r - f, r, (double) (r - f) * 100.0 / r);
+		cw->cw_printf(cw, " Packets/bytes/errors:\n");
+		cw->cw_printf(cw, "  Ingress:   %10" PRIu64 " / %10" PRIu64 " / %10" PRIu64 "\n",
+				atomic64_get_na(&lif->stats->in.packets),
+				atomic64_get_na(&lif->stats->in.bytes),
+				atomic64_get_na(&lif->stats->in.errors));
+		cw->cw_printf(cw, "  Egress:    %10" PRIu64 " / %10" PRIu64 " / %10" PRIu64 "\n",
+				atomic64_get_na(&lif->stats->out.packets),
+				atomic64_get_na(&lif->stats->out.bytes),
+				atomic64_get_na(&lif->stats->out.errors));
+		cw->cw_printf(cw, " Packets lost/duplicates: %10" PRIu64 " / %10" PRIu64 "\n",
+				atomic64_get_na(&lif->stats->s.packets_lost),
+				atomic64_get_na(&lif->stats->s.duplicates));
+		cw->cw_printf(cw, " MOS:    avg %3.1f, packet loss avg %3.0f%%\n",
+				(double) atomic64_get_na(&lif->stats->sampled.sums.mos)
+					/ atomic64_get_na(&lif->stats->sampled.counts.mos) / 10.,
+				(double) atomic64_get_na(&lif->stats->sampled.sums.packetloss)
+					/ atomic64_get_na(&lif->stats->sampled.counts.packetloss));
+		cw->cw_printf(cw, " Jitter: avg %3.0f (measured %3.0f)\n",
+				(double) atomic64_get_na(&lif->stats->sampled.sums.jitter)
+					/ atomic64_get_na(&lif->stats->sampled.counts.jitter),
+				(double) atomic64_get_na(&lif->stats->sampled.sums.jitter_measured)
+					/ atomic64_get_na(&lif->stats->sampled.counts.jitter_measured));
+		cw->cw_printf(cw, " RTT:    e2e %3.0f, dsct %3.0f\n",
+				(double) atomic64_get_na(&lif->stats->sampled.sums.rtt_e2e)
+					/ atomic64_get_na(&lif->stats->sampled.counts.rtt_e2e),
+				(double) atomic64_get_na(&lif->stats->sampled.sums.rtt_dsct)
+					/ atomic64_get_na(&lif->stats->sampled.counts.rtt_dsct));
 	}
 }
 
 static void cli_incoming_list_jsonstats(str *instr, struct cli_writer *cw) {
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics, statistics_gather_metrics(NULL));
+	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(NULL);
 
-	for (GList *l = metrics->head; l; l = l->next) {
-		struct stats_metric *m = l->data;
+	for (__auto_type l = metrics->head; l; l = l->next) {
+		stats_metric *m = l->data;
 		if (!m->label)
 			continue;
 
@@ -1705,15 +1701,17 @@ static void cli_incoming_list_jsonstats(str *instr, struct cli_writer *cw) {
 static void cli_incoming_list_transcoders(str *instr, struct cli_writer *cw) {
 	mutex_lock(&rtpe_codec_stats_lock);
 
-	GList *chains = g_hash_table_get_keys(rtpe_codec_stats);
-	if (!chains)
+	if (t_hash_table_size(rtpe_codec_stats) == 0)
 		cw->cw_printf(cw, "No stats entries\n");
 	else {
 		int last_tv_sec = rtpe_now.tv_sec - 1;
 		unsigned int idx = last_tv_sec & 1;
-		for (GList *l = chains; l; l = l->next) {
-			char *chain = l->data;
-			struct codec_stats *stats_entry = g_hash_table_lookup(rtpe_codec_stats, chain);
+
+		codec_stats_ht_iter iter;
+		t_hash_table_iter_init(&iter, rtpe_codec_stats);
+		char *chain;
+		struct codec_stats *stats_entry;
+		while (t_hash_table_iter_next(&iter, &chain, &stats_entry)) {
 			cw->cw_printf(cw, "%s: %i transcoders\n", chain, g_atomic_int_get(&stats_entry->num_transcoders));
 			if (g_atomic_int_get(&stats_entry->last_tv_sec[idx]) != last_tv_sec)
 				continue;
@@ -1724,14 +1722,10 @@ static void cli_incoming_list_transcoders(str *instr, struct cli_writer *cw) {
 	}
 
 	mutex_unlock(&rtpe_codec_stats_lock);
-
-	g_list_free(chains);
 }
 
 static void cli_incoming_list_controltos(str *instr, struct cli_writer *cw) {
-	rwlock_lock_r(&rtpe_config.config_lock);
-	cw->cw_printf(cw, "%d\n", rtpe_config.control_tos);
-	rwlock_unlock_r(&rtpe_config.config_lock);
+	cw->cw_printf(cw, "%d\n", atomic_get_na(&rtpe_config.control_tos));
 }
 
 static void cli_incoming_set_controltos(str *instr, struct cli_writer *cw) {
@@ -1750,9 +1744,7 @@ static void cli_incoming_set_controltos(str *instr, struct cli_writer *cw) {
 		return;
 	}
 
-	rwlock_lock_w(&rtpe_config.config_lock);
-	rtpe_config.control_tos = tos;
-	rwlock_unlock_w(&rtpe_config.config_lock);
+	atomic_set_na(&rtpe_config.control_tos, tos);
 
 	for (GList *l = rtpe_control_ng.head; l; l = l->next) {
 		struct control_ng *c = l->data;
