@@ -5,8 +5,6 @@
 #include <openssl/hmac.h>
 #include <glib.h>
 
-#include "xt_RTPENGINE.h"
-
 #include "str.h"
 #include "helpers.h"
 #include "rtp.h"
@@ -16,30 +14,28 @@
 #include "rtcplib.h"
 #include "main.h"
 #include "ssllib.h"
+#include "types.h"
 
-
-
+#include "xt_RTPENGINE.h"
 
 __thread GString *crypto_debug_string;
 
-
-
-static int aes_cm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
-static int aes_cm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
-static int aes_gcm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
-static int aes_gcm_decrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint64_t);
-static int aes_gcm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
-static int aes_gcm_decrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint64_t);
-static int hmac_sha1_rtp(struct crypto_context *, char *out, str *in, uint64_t);
+static int aes_cm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint32_t);
+static int aes_cm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint32_t);
+static int aes_gcm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint32_t);
+static int aes_gcm_decrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint32_t);
+static int aes_gcm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint32_t);
+static int aes_gcm_decrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint32_t);
+static int hmac_sha1_rtp(struct crypto_context *, char *out, str *in, uint32_t);
 static int hmac_sha1_rtcp(struct crypto_context *, char *out, str *in);
-static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx);
-static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx);
+static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx);
+static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx);
 static int aes_cm_session_key_init(struct crypto_context *c);
 static int aes_gcm_session_key_init(struct crypto_context *c);
 static int aes_f8_session_key_init(struct crypto_context *c);
 static int evp_session_key_cleanup(struct crypto_context *c);
-static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx);
-static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx);
+static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx);
+static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx);
 
 /* all lengths are in bytes */
 struct crypto_suite __crypto_suites[] = {
@@ -337,22 +333,14 @@ const unsigned int num_crypto_suites = G_N_ELEMENTS(__crypto_suites);
 
 
 const struct crypto_suite * crypto_find_suite(const str *s) {
-	int i, l;
+	int i;
 	const struct crypto_suite *cs;
 
 	for (i = 0; i < num_crypto_suites; i++) {
 		cs = &crypto_suites[i];
-		if (!cs->name)
-			continue;
 
-		l = strlen(cs->name);
-		if (l != s->len)
-			continue;
-
-		if (strncasecmp(cs->name, s->s, s->len))
-			continue;
-
-		return cs;
+		if (str_casecmp_str(&cs->name_str, s) == 0)
+			return cs;
 	}
 
 	return NULL;
@@ -527,7 +515,7 @@ int crypto_gen_session_key(struct crypto_context *c, str *out, unsigned char lab
  */
 
 /* rfc 3711 section 4.1.1 */
-static int aes_cm_encrypt(struct crypto_context *c, uint32_t ssrc, str *s, uint64_t idx) {
+static int aes_cm_encrypt(struct crypto_context *c, uint32_t ssrc, str *s, uint32_t idx) {
 	unsigned char iv[16];
 	uint32_t *ivi;
 	uint32_t idxh, idxl;
@@ -535,9 +523,8 @@ static int aes_cm_encrypt(struct crypto_context *c, uint32_t ssrc, str *s, uint6
 	memcpy(iv, c->session_salt, 14);
 	iv[14] = iv[15] = '\0';
 	ivi = (void *) iv;
-	idx <<= 16;
-	idxh = htonl((idx & 0xffffffff00000000ULL) >> 32);
-	idxl = htonl(idx & 0xffffffffULL);
+	idxh = htonl((idx & 0xffff0000ULL) >> 16);
+	idxl = htonl((idx & 0x0000ffffULL) << 16);
 
 	ivi[1] ^= ssrc;
 	ivi[2] ^= idxh;
@@ -549,12 +536,12 @@ static int aes_cm_encrypt(struct crypto_context *c, uint32_t ssrc, str *s, uint6
 }
 
 /* rfc 3711 section 4.1 */
-static int aes_cm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+static int aes_cm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx) {
 	return aes_cm_encrypt(c, r->ssrc, s, idx);
 }
 
 /* rfc 3711 sections 3.4 and 4.1 */
-static int aes_cm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+static int aes_cm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx) {
 	return aes_cm_encrypt(c, r->ssrc, s, idx);
 }
 
@@ -573,15 +560,15 @@ union aes_gcm_rtp_iv {
 _Static_assert(offsetof(union aes_gcm_rtp_iv, seq) == 10,
                "union aes_gcm_rtp_iv not packed");
 
-static int aes_gcm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+static int aes_gcm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx) {
 	union aes_gcm_rtp_iv iv;
 	int len, ciphertext_len;
 
 	memcpy(iv.bytes, c->session_salt, 12);
 
 	iv.ssrc ^= r->ssrc;
-	iv.roq ^= htonl((idx & 0x00ffffffff0000ULL) >> 16);
-	iv.seq ^= htons(idx & 0x00ffffULL);
+	iv.roq ^= htonl((idx & 0xffffff0000ULL) >> 16);
+	iv.seq ^= htons( idx & 0x000000ffffULL);
 
 	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
 			(const unsigned char *) c->session_key, iv.bytes);
@@ -602,7 +589,7 @@ static int aes_gcm_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, s
 	return 0;
 }
 
-static int aes_gcm_decrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+static int aes_gcm_decrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx) {
 	union aes_gcm_rtp_iv iv;
 	int len, plaintext_len;
 
@@ -612,8 +599,8 @@ static int aes_gcm_decrypt_rtp(struct crypto_context *c, struct rtp_header *r, s
 	memcpy(iv.bytes, c->session_salt, 12);
 
 	iv.ssrc ^= r->ssrc;
-	iv.roq ^= htonl((idx & 0x00ffffffff0000ULL) >> 16);
-	iv.seq ^= htons(idx & 0x00ffffULL);
+	iv.roq ^= htonl((idx & 0xffff0000ULL) >> 16);
+	iv.seq ^= htons( idx & 0x0000ffffULL);
 
 	EVP_DecryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aead_evp(), NULL,
 			(const unsigned char *) c->session_key, iv.bytes);
@@ -649,7 +636,7 @@ union aes_gcm_rtcp_iv {
 _Static_assert(offsetof(union aes_gcm_rtcp_iv, srtcp) == 8,
                "union aes_gcm_rtcp_iv not packed");
 
-static int aes_gcm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+static int aes_gcm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx) {
 	union aes_gcm_rtcp_iv iv;
 	uint32_t e_idx;
 	int len, ciphertext_len;
@@ -680,7 +667,7 @@ static int aes_gcm_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r,
 	return 0;
 }
 
-static int aes_gcm_decrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+static int aes_gcm_decrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx) {
 	union aes_gcm_rtcp_iv iv;
 	uint32_t e_idx;
 	int len, plaintext_len;
@@ -776,7 +763,7 @@ done:
 }
 
 /* rfc 3711 section 4.1.2.2 */
-static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx) {
 	unsigned char iv[16];
 	uint32_t roc;
 
@@ -791,7 +778,7 @@ static int aes_f8_encrypt_rtp(struct crypto_context *c, struct rtp_header *r, st
 }
 
 /* rfc 3711 section 4.1.2.3 */
-static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx) {
 	unsigned char iv[16];
 	uint32_t i;
 
@@ -805,7 +792,7 @@ static int aes_f8_encrypt_rtcp(struct crypto_context *c, struct rtcp_packet *r,
 	return 0;
 }
 /* rfc 3711, sections 4.2 and 4.2.1 */
-static int hmac_sha1_rtp(struct crypto_context *c, char *out, str *in, uint64_t index) {
+static int hmac_sha1_rtp(struct crypto_context *c, char *out, str *in, uint32_t index) {
 	unsigned char hmac[20];
 	uint32_t roc;
 
@@ -945,10 +932,10 @@ static int evp_session_key_cleanup(struct crypto_context *c) {
 	return 0;
 }
 
-static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint64_t idx) {
+static int null_crypt_rtp(struct crypto_context *c, struct rtp_header *r, str *s, uint32_t idx) {
 	return 0;
 }
-static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint64_t idx) {
+static int null_crypt_rtcp(struct crypto_context *c, struct rtcp_packet *r, str *s, uint32_t idx) {
 	return 0;
 }
 
@@ -1005,7 +992,7 @@ out:
 	return *buf;
 }
 
-void crypto_init_main() {
+void crypto_init_main(void) {
 	struct crypto_suite *cs;
 	for (unsigned int i = 0; i < num_crypto_suites; i++) {
 		cs = &__crypto_suites[i];
