@@ -1,8 +1,10 @@
 #include "log.h"
+
 #include <syslog.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <glib.h>
+
 #include "str.h"
 #include "call.h"
 #include "poller.h"
@@ -10,9 +12,6 @@
 #include "loglib.h"
 #include "main.h"
 
-
-
-
 __thread struct log_info log_info;
 __thread GSList *log_info_stack;
 
@@ -166,7 +165,7 @@ void rtcplog(const char* cdrbuffer) {
 }
 
 int get_local_log_level(unsigned int subsystem_idx) {
-	struct call *call = NULL;
+	call_t *call = NULL;
 
 	switch (log_info.e) {
 		case LOG_INFO_CALL:
