@@ -26,7 +26,11 @@ struct streambuf_callback {
 	struct obj *parent;
 };
 
-static void tcp_listener_incoming(int fd, void *p, uintptr_t x) {
+TYPED_DIRECT_FUNCS(tcp_direct_hash, tcp_direct_eq, struct streambuf_stream)
+TYPED_GHASHTABLE_IMPL(tcp_streams_ht, tcp_direct_hash, tcp_direct_eq, NULL, NULL)
+
+
+static void tcp_listener_incoming(int fd, void *p) {
 	struct tcp_listener_callback *cb = p;
 	int ret;
 	char addr[64];
@@ -54,8 +58,9 @@ static void tcp_listener_incoming(int fd, void *p, uintptr_t x) {
 	}
 }
 
-static void tcp_listener_closed(int fd, void *p, uintptr_t u) {
-	abort();
+static void tcp_listener_closed(int fd, void *p) {
+	if (!rtpe_shutdown)
+		abort();
 }
 
 static void __tlc_free(void *p) {
@@ -63,7 +68,7 @@ static void __tlc_free(void *p) {
 	obj_put_o(cb->p);
 }
 
-int tcp_listener_init(socket_t *sock, const endpoint_t *ep,
+static int tcp_listener_init(socket_t *sock, const endpoint_t *ep,
 		tcp_listener_callback_t func, struct obj *obj)
 {
 	struct poller_item i;
@@ -84,7 +89,7 @@ int tcp_listener_init(socket_t *sock, const endpoint_t *ep,
 	i.closed = tcp_listener_closed;
 	i.readable = tcp_listener_incoming;
 	i.obj = &cb->obj;
-	if (poller_add_item(rtpe_poller, &i))
+	if (!rtpe_poller_add_item(rtpe_control_poller, &i))
 		goto fail;
 
 	obj_put(cb);
@@ -98,7 +103,6 @@ fail:
 
 static void streambuf_stream_free(void *p) {
 	struct streambuf_stream *s = p;
-	close_socket(&s->sock);
 	streambuf_destroy(s->inbuf);
 	streambuf_destroy(s->outbuf);
 	obj_put(s->cb);
@@ -106,7 +110,7 @@ static void streambuf_stream_free(void *p) {
 	free(s->addr);
 }
 
-static void streambuf_stream_closed(int fd, void *p, uintptr_t u) {
+static void streambuf_stream_closed(int fd, void *p) {
 	struct streambuf_stream *s = p;
 
 	if (s->sock.fd == -1)
@@ -117,14 +121,15 @@ static void streambuf_stream_closed(int fd, void *p, uintptr_t u) {
 
 	struct streambuf_listener *l = s->listener;
 	mutex_lock(&l->lock);
-	int ret = g_hash_table_remove(l->streams, s);
+	bool ret = t_hash_table_remove(l->streams, s);
 	mutex_unlock(&l->lock);
-	poller_del_item(rtpe_poller, s->sock.fd);
+	rtpe_poller_del_item(rtpe_control_poller, s->sock.fd);
+	reset_socket(&s->sock);
 	if (ret)
 		obj_put(s);
 }
 
-static void streambuf_stream_readable(int fd, void *p, uintptr_t u) {
+static void streambuf_stream_readable(int fd, void *p) {
 	struct streambuf_stream *s = p;
 
 	int ret = streambuf_readable(s->inbuf);
@@ -139,14 +144,14 @@ static void streambuf_stream_readable(int fd, void *p, uintptr_t u) {
 	return;
 
 close:
-	streambuf_stream_closed(fd, s, 0);
+	streambuf_stream_closed(fd, s);
 }
 
-static void streambuf_stream_writeable(int fd, void *p, uintptr_t u) {
+static void streambuf_stream_writeable(int fd, void *p) {
 	struct streambuf_stream *s = p;
 
 	if (streambuf_writeable(s->outbuf))
-		streambuf_stream_closed(fd, s, 0);
+		streambuf_stream_closed(fd, s);
 }
 
 
@@ -160,8 +165,8 @@ static void streambuf_listener_newconn(struct obj *p, socket_t *newsock, char *a
 
 	s = obj_alloc0("streambuf_stream", sizeof(*s), streambuf_stream_free);
 	s->sock = *newsock;
-	s->inbuf = streambuf_new(rtpe_poller, newsock->fd);
-	s->outbuf = streambuf_new(rtpe_poller, newsock->fd);
+	s->inbuf = streambuf_new(rtpe_control_poller, newsock->fd);
+	s->outbuf = streambuf_new(rtpe_control_poller, newsock->fd);
 	s->listener = listener;
 	s->cb = obj_get(cb);
 	s->parent = obj_get_o(cb->parent);
@@ -180,10 +185,10 @@ static void streambuf_listener_newconn(struct obj *p, socket_t *newsock, char *a
 	obj_hold(s);
 
 	mutex_lock(&listener->lock);
-	g_hash_table_insert(listener->streams, s, s); // hand over ref
+	t_hash_table_insert(listener->streams, s, s); // hand over ref
 	mutex_unlock(&listener->lock);
 
-	if (poller_add_item(rtpe_poller, &i))
+	if (!rtpe_poller_add_item(rtpe_control_poller, &i))
 		goto fail;
 
 	obj_put(s);
@@ -192,7 +197,7 @@ static void streambuf_listener_newconn(struct obj *p, socket_t *newsock, char *a
 
 fail:
 	mutex_lock(&listener->lock);
-	int ret = g_hash_table_remove(listener->streams, s);
+	bool ret = t_hash_table_remove(listener->streams, s);
 	mutex_unlock(&listener->lock);
 
 	if (ret)
@@ -217,7 +222,7 @@ int streambuf_listener_init(struct streambuf_listener *listener, const endpoint_
 	ZERO(*listener);
 
 	mutex_init(&listener->lock);
-	listener->streams = g_hash_table_new(g_direct_hash, g_direct_equal);
+	listener->streams = tcp_streams_ht_new();
 
 	cb = obj_alloc("streambuf_callback", sizeof(*cb), __sb_free);
 	cb->newconn_func = newconn_func;
@@ -239,14 +244,13 @@ fail:
 void streambuf_listener_shutdown(struct streambuf_listener *listener) {
 	if (!listener)
 		return;
-	poller_del_item(rtpe_poller, listener->listener.fd);
-	close_socket(&listener->listener);
-	if (listener->streams)
-		g_hash_table_destroy(listener->streams);
+	rtpe_poller_del_item(rtpe_control_poller, listener->listener.fd);
+	reset_socket(&listener->listener);
+	t_hash_table_destroy_ptr(&listener->streams);
 }
 
 void streambuf_stream_close(struct streambuf_stream *s) {
-	streambuf_stream_closed(s->sock.fd, s, 0);
+	streambuf_stream_closed(s->sock.fd, s);
 }
 void streambuf_stream_shutdown(struct streambuf_stream *s) {
 	shutdown(s->sock.fd, SHUT_WR);
