@@ -4,6 +4,6 @@
 extern int load_average; // times 100
 extern int cpu_usage; // times 100
 
-void load_thread(void *);
+enum thread_looper_action load_thread(void);
 
 #endif
