@@ -1,31 +1,23 @@
 #ifndef _SSRC_H_
 #define _SSRC_H_
 
-
 #include <sys/types.h>
 #include <glib.h>
+
 #include "compat.h"
 #include "helpers.h"
 #include "obj.h"
 #include "codeclib.h"
+#include "types.h"
 
-
-
-
-struct call;
 struct call_media;
 struct timeval;
-struct rtp_payload_type;
 struct ssrc_entry;
 struct ssrc_entry_call;
 enum ssrc_dir;
-struct stream_fd;
-
-
 
 typedef struct ssrc_entry *(*ssrc_create_func_t)(void *uptr);
 
-
 struct ssrc_hash {
 	GHashTable *ht;
 	GQueue q;
@@ -49,19 +41,14 @@ struct ssrc_ctx {
 	struct payload_tracker tracker;
 	void *ref; // points to the call_monologue but is opaque
 
-	// XXX lock this?
-	uint64_t srtp_index,
-		  srtcp_index;
 	// XXX move entire crypto context in here?
 
 	// for transcoding
 	uint32_t ssrc_map_out;
+	uint16_t seq_out;
 
 	// RTCP stats
-	atomic64 packets,
-		 octets,
-		 last_seq, // XXX dup with srtp_index?
-		 last_ts;
+	struct ssrc_stats *stats;
 
 	// for per-second stats:
 	atomic64 last_sample,
@@ -203,6 +190,15 @@ struct ssrc_xr_voip_metrics {
 	uint16_t jb_abs_max;
 };
 
+struct crtt_args {
+	struct ssrc_hash *ht;
+	const struct timeval *tv;
+	int * pt_p;
+	uint32_t ssrc;
+	uint32_t ntp_middle_bits;
+	uint32_t delay;
+	size_t reports_queue_offset;
+};
 
 
 
@@ -222,7 +218,7 @@ struct ssrc_ctx *get_ssrc_ctx(uint32_t, struct ssrc_hash *, enum ssrc_dir, void
 
 
 void ssrc_sender_report(struct call_media *, const struct ssrc_sender_report *, const struct timeval *);
-void ssrc_receiver_report(struct call_media *, struct stream_fd *, const struct ssrc_receiver_report *,
+void ssrc_receiver_report(struct call_media *, stream_fd *, const struct ssrc_receiver_report *,
 		const struct timeval *);
 void ssrc_receiver_rr_time(struct call_media *m, const struct ssrc_xr_rr_time *rr,
 		const struct timeval *);
