@@ -8,6 +8,7 @@
 #include <netinet/udp.h>
 #include <sys/socket.h>
 #include "str.h"
+#include "auxlib.h"
 #include "xt_RTPENGINE.h"
 #include "log.h"
 
@@ -320,7 +321,6 @@ INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoin
 	struct msghdr msg;
 	struct iovec iov;
 	char ctrl[64];
-	struct cmsghdr *cm;
 
 	ZERO(msg);
 	msg.msg_name = &sin;
@@ -338,28 +338,7 @@ INLINE ssize_t __ip_recvfrom_options(socket_t *s, void *buf, size_t len, endpoin
 		return ret;
 	s->family->sockaddr2endpoint(ep, &sin);
 
-	if (tv || to) {
-		for (cm = CMSG_FIRSTHDR(&msg); cm; cm = CMSG_NXTHDR(&msg, cm)) {
-			if (cm->cmsg_level == SOL_SOCKET && cm->cmsg_type == SO_TIMESTAMP && tv) {
-				*tv = *((struct timeval *) CMSG_DATA(cm));
-				tv = NULL;
-			}
-			if (parse && to && parse(cm, to))
-				to = NULL;
-		}
-		if (G_UNLIKELY(tv)) {
-			ilog(LOG_WARNING, "No receive timestamp received from kernel");
-			ZERO(*tv);
-		}
-		if (G_UNLIKELY(to)) {
-			ilog(LOG_WARNING, "No local address received from kernel");
-			ZERO(*to);
-		}
-	}
-	if (G_UNLIKELY((msg.msg_flags & MSG_TRUNC)))
-		ilog(LOG_WARNING, "Kernel indicates that data was truncated");
-	if (G_UNLIKELY((msg.msg_flags & MSG_CTRUNC)))
-		ilog(LOG_WARNING, "Kernel indicates that ancillary data was truncated");
+	socket_recvfrom_parse_cmsg(&tv, &to, parse, &msg, CMSG_FIRSTHDR(&msg), CMSG_NXTHDR(&msg, cm));
 
 	return ret;
 }
@@ -563,15 +542,9 @@ gint sockaddr_t_eq(gconstpointer a, gconstpointer b) {
 unsigned int endpoint_hash(const endpoint_t *a) {
 	return sockaddr_hash(&a->address) ^ a->port;
 }
-bool endpoint_eq(const endpoint_t *a, const endpoint_t *b) {
+gboolean endpoint_eq(const endpoint_t *a, const endpoint_t *b) {
 	return sockaddr_eq(&a->address, &b->address) && a->port == b->port;
 }
-guint endpoint_t_hash(gconstpointer a) {
-	return endpoint_hash(a);
-}
-gint endpoint_t_eq(gconstpointer a, const void *b) {
-	return endpoint_eq(a, b);
-}
 
 
 
@@ -874,6 +847,16 @@ int connect_socket_nb(socket_t *r, int type, const endpoint_t *ep) {
 	return connect_socket_retry(r);
 }
 
+int reset_socket(socket_t *r) {
+	if (!r)
+		return -1;
+
+	r->fd = -1;
+	ZERO(r->local);
+	ZERO(r->remote);
+
+	return 0;
+}
 int close_socket(socket_t *r) {
 	if (!r) {
 		__C_DBG("close() syscall not called, no socket");
@@ -891,9 +874,7 @@ int close_socket(socket_t *r) {
 
 	__C_DBG("close() syscall success, fd=%d", r->fd);
 
-	r->fd = -1;
-	ZERO(r->local);
-	ZERO(r->remote);
+	reset_socket(r);
 
 	return 0;
 }
@@ -904,9 +885,7 @@ int close_socket(socket_t *r) {
 // does not actually close the socket
 void move_socket(socket_t *dst, socket_t *src) {
 	*dst = *src;
-	src->fd = -1;
-	ZERO(src->local);
-	ZERO(src->remote);
+	reset_socket(src);
 }
 
 
