@@ -32,7 +32,7 @@ decode_t *decoder_new(const char *payload_str, const char *format, int ptime, ou
 		return NULL;
 	}
 
-	str name = STR_INIT_LEN((char *) payload_str, slash - payload_str);
+	str name = STR_INIT_LEN(payload_str, slash - payload_str);
 	int clockrate = atoi(slash + 1);
 	if (clockrate <= 0) {
 		ilog(LOG_ERR, "Invalid clock rate %i (parsed from '%.20s'/'%.20s')",
@@ -82,7 +82,7 @@ decode_t *decoder_new(const char *payload_str, const char *format, int ptime, ou
 	else
 		out_format.format = AV_SAMPLE_FMT_S16; // needed for TLS-only scenarios
 
-	str fmtp = STR_INIT((char*) format);
+	str fmtp = STR_INIT(format);
 
 	decoder_t *dec = decoder_new_fmtp(def, rtp_clockrate, channels, ptime, &out_format, NULL, &fmtp, NULL);
 	if (!dec)
@@ -115,7 +115,7 @@ static int decoder_got_frame(decoder_t *dec, AVFrame *frame, void *sp, void *dp)
 	if (metafile->mix_out) {
 		dbg("adding packet from stream #%lu to mix output", stream->id);
 		if (G_UNLIKELY(deco->mixer_idx == (unsigned int) -1))
-			deco->mixer_idx = mix_get_index(metafile->mix, ssrc);
+			deco->mixer_idx = mix_get_index(metafile->mix, ssrc, stream->media_sdp_id);
 		format_t actual_format;
 		if (output_config(metafile->mix_out, &dec->dest_format, &actual_format))
 			goto no_mix_out;
